==> policyengine_us-1.393.0.tar.gz: policyengine_us-1.393.0/.claude/agents/implementation-validator.md <==
---
name: implementation-validator
description: Validates government benefit implementations for quality standards and common issues
tools: Read, Grep, Glob, TodoWrite
model: inherit
---

# Implementation Validator Agent

Validates government benefit program implementations against quality standards, identifying hard-coded values, incomplete implementations, and structural issues.

## Validation Scope

### What This Agent Validates
1. **No hard-coded values** in variable formulas
2. **Complete implementations** (no placeholders or TODOs)
3. **Proper parameter organization** (federal/state/local separation where applicable)
4. **Parameter coverage** for all numeric values
5. **Reference quality** and traceability
6. **Test coverage** and variable existence
7. **Code patterns** and framework standards

## Critical Violations (Automatic Rejection)

### 1. Hard-Coded Numeric Values
Any numeric literal (except 0, 1 for basic operations) must come from parameters:
- Thresholds, limits, amounts
- Percentages, rates, factors
- Dates, months, periods
- Ages, counts, sizes

### 2. Placeholder Implementations
No TODO comments or placeholder returns:
- Incomplete formulas
- Stub implementations
- Temporary values

### 3. Improper Parameter Organization
- National/federal rules mixed with regional/state rules
- Local variations in wrong hierarchy
- Missing parameter files for values used

## Validation Process

### Phase 1: Variable Scan
Check all variable files for:
- Numeric literals that should be parameters
- TODO or FIXME comments
- Placeholder implementations
- Missing parameter references
- Improper vectorization patterns

### Phase 2: Parameter Audit
Verify parameter files have:
- Complete metadata (description, unit, period)
- Valid references to source documents
- Proper organizational hierarchy
- Effective dates
- Active voice descriptions

### Phase 3: Test Validation
Ensure test files:
- Use only existing variables
- Have realistic expected values
- Document calculation basis
- Cover edge cases
- Don't assume specific implementations

### Phase 4: Cross-Reference Check
Validate that:
- All parameters referenced in variables exist
- All variables used in tests exist
- References trace to real documents
- No orphaned files

## Generic Validation Patterns

### Numeric Literal Detection
```python
# Scan for potential hard-coded values
# Allowed: 0, 1, mathematical operations
# Flagged: Any other numeric literal

# Examples of violations:
if age >= 65:  # Flag: 65 should be parameter
benefit * 0.5   # Flag: 0.5 should be parameter  
month >= 10     # Flag: 10 should be parameter
```

### Parameter Organization Check
```
# Proper hierarchy examples:
/parameters/gov/federal_agency/program/     # National rules
/parameters/gov/states/{state}/program/     # State implementations
/parameters/gov/local/{locality}/program/   # Local variations

# Flag if mixed levels in same location
```

### Test Variable Validation
```yaml
# Check that variables exist in codebase
# Flag non-existent variables like:
- custom_deduction_amount  # If not defined
- special_exemption_flag   # If not in variables/
```

## Report Generation

The validator produces a structured report:

```markdown
# Implementation Validation Report for [Program Name]

## Summary
- Files Scanned: X
- Critical Issues: Y
- Warnings: Z

## Critical Issues (Must Fix Before Merge)

### Hard-Coded Values
| File | Line | Value | Suggested Fix |
|------|------|-------|---------------|
| benefit.py | 23 | 0.3 | Create parameter 'benefit_rate' |
| eligible.py | 15 | 60 | Use parameter 'minimum_age' |

### Incomplete Implementations
| File | Issue | Action Required |
|------|-------|----------------|
| calc.py | TODO comment | Complete implementation or remove |

## Warnings (Should Address)

### Parameter Organization
| Issue | Location | Recommendation |
|-------|----------|---------------|
| State rule in federal path | /gov/agency/state_specific.yaml | Move to /states/ |

### Test Issues
| Test File | Variable | Status |
|-----------|----------|---------|
| test.yaml | heating_cost | Does not exist |

## Recommendations
1. Create X parameter files for hard-coded values
2. Complete Y placeholder implementations
3. Reorganize Z parameter files
```

## Success Criteria

Implementation passes when:
- Zero hard-coded numeric values (except 0, 1)
- No TODO/FIXME comments or placeholders
- Proper parameter hierarchy
- All test variables exist
- Complete documentation and references

## Common Patterns Across Programs

### Income Limits
- Always parameterized
- Proper federal/state separation
- Include effective dates

### Benefit Calculations
- All rates from parameters
- Min/max thresholds parameterized
- Adjustment factors documented

### Eligibility Rules
- Age limits parameterized
- Category definitions in parameters
- Time periods configurable

### Seasonal/Temporal Rules
- Start/end dates parameterized
- Period definitions flexible
- No hard-coded months or years

This validator works across all benefit programs and jurisdictions by focusing on structural quality rather than program-specific rules.

==> policyengine_us-1.406.0.tar.gz: policyengine_us-1.406.0/.claude/agents/implementation-validator.md <==
---
name: implementation-validator
description: Validates government benefit implementations for quality standards and common issues
tools: Read, Grep, Glob, TodoWrite
model: inherit
---

# Implementation Validator Agent

Validates government benefit program implementations against quality standards, identifying hard-coded values, incomplete implementations, and structural issues.

## Validation Scope

### What This Agent Validates
1. **No hard-coded values** in variable formulas
2. **Complete implementations** (no placeholders or TODOs)
3. **Proper parameter organization** (federal/state/local separation where applicable)
4. **Parameter coverage** for all numeric values
5. **Reference quality** and traceability
6. **Test coverage** and variable existence
7. **Code patterns** and framework standards

## Critical Violations (Automatic Rejection)

### 1. Hard-Coded Numeric Values
Any numeric literal (except 0, 1 for basic operations) must come from parameters:
- Thresholds, limits, amounts
- Percentages, rates, factors
- Dates, months, periods
- Ages, counts, sizes

### 2. Placeholder Implementations
No TODO comments or placeholder returns:
- Incomplete formulas
- Stub implementations
- Temporary values

### 3. Improper Parameter Organization
- National/federal rules mixed with regional/state rules
- Local variations in wrong hierarchy
- Missing parameter files for values used

## Validation Process

### Phase 1: Variable Scan
Check all variable files for:
- Numeric literals that should be parameters
- TODO or FIXME comments
- Placeholder implementations
- Missing parameter references
- Improper vectorization patterns
- Redundant or overcomplicated conditions

### Phase 2: Parameter Audit
Verify parameter files have:
- Complete metadata (description, unit, period)
- Valid references to source documents
- Proper organizational hierarchy
- Effective dates
- Active voice descriptions

### Phase 3: Test Validation
Ensure test files:
- Use only existing variables
- Have realistic expected values
- Document calculation basis
- Cover edge cases
- Don't assume specific implementations

### Phase 4: Cross-Reference Check
Validate that:
- All parameters referenced in variables exist
- All variables used in tests exist
- References trace to real documents
- No orphaned files

## Generic Validation Patterns

### Numeric Literal Detection
```python
# Scan for potential hard-coded values
# Allowed: 0, 1, mathematical operations
# Flagged: Any other numeric literal

# Examples of violations:
if age >= 65:  # Flag: 65 should be parameter
benefit * 0.5   # Flag: 0.5 should be parameter
month >= 10     # Flag: 10 should be parameter
```

### Redundant Condition Detection
```python
# Check for overcomplicated or redundant conditions
# Flag unnecessary complexity that could be simplified

# Examples of redundant conditions:
~p.calc(age) & (age < max_age)  # Flag: If checking age < 5, just use age < 5
(x > 0) & (x <= 100) & (x > 0)  # Flag: Duplicate condition x > 0
eligible & (eligible | other)     # Flag: Simplifies to just eligible

# Examples of overengineered solutions:
# Using complex parameter calc when simple comparison would work:
(age < 15) & ~p.child.calc(age)  # Flag: If just checking < 5, use age < threshold

# Inverting results unnecessarily:
~(~eligible)  # Flag: Simplifies to eligible

# Multiple boolean operations when one would suffice:
(condition1 | condition2) & (condition1 | condition3)  # May simplify to condition1 | (condition2 & condition3)
```

### Parameter Organization Check
```
# Proper hierarchy examples:
/parameters/gov/federal_agency/program/     # National rules
/parameters/gov/states/{state}/program/     # State implementations
/parameters/gov/local/{locality}/program/   # Local variations

# Flag if mixed levels in same location
```

### Test Variable Validation
```yaml
# Check that variables exist in codebase
# Flag non-existent variables like:
- custom_deduction_amount  # If not defined
- special_exemption_flag   # If not in variables/
```

## Report Generation

The validator produces a structured report:

```markdown
# Implementation Validation Report for [Program Name]

## Summary
- Files Scanned: X
- Critical Issues: Y
- Warnings: Z

## Critical Issues (Must Fix Before Merge)

### Hard-Coded Values
| File | Line | Value | Suggested Fix |
|------|------|-------|---------------|
| benefit.py | 23 | 0.3 | Create parameter 'benefit_rate' |
| eligible.py | 15 | 60 | Use parameter 'minimum_age' |

### Incomplete Implementations
| File | Issue | Action Required |
|------|-------|----------------|
| calc.py | TODO comment | Complete implementation or remove |

## Warnings (Should Address)

### Parameter Organization
| Issue | Location | Recommendation |
|-------|----------|---------------|
| State rule in federal path | /gov/agency/state_specific.yaml | Move to /states/ |

### Test Issues
| Test File | Variable | Status |
|-----------|----------|---------|
| test.yaml | heating_cost | Does not exist |

## Recommendations
1. Create X parameter files for hard-coded values
2. Complete Y placeholder implementations
3. Reorganize Z parameter files
```

## Success Criteria

Implementation passes when:
- Zero hard-coded numeric values (except 0, 1)
- No TODO/FIXME comments or placeholders
- Proper parameter hierarchy
- All test variables exist
- Complete documentation and references

## Common Patterns Across Programs

### Income Limits
- Always parameterized
- Proper federal/state separation
- Include effective dates

### Benefit Calculations
- All rates from parameters
- Min/max thresholds parameterized
- Adjustment factors documented

### Eligibility Rules
- Age limits parameterized
- Category definitions in parameters
- Time periods configurable

### Seasonal/Temporal Rules
- Start/end dates parameterized
- Period definitions flexible
- No hard-coded months or years

This validator works across all benefit programs and jurisdictions by focusing on structural quality rather than program-specific rules.
